#!/usr/bin/env python3
import queue
import threading
import time
import tkinter
import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime
from tkinter.constants import RAISED
from statusd_server.converters import Converter
from statusd_server.models import Cpu

from statusd_server import server


class Gui:
    def __init__(self):
        self.queue = queue.Queue()

    def run(self):
        queueThread = threading.Thread(target=self.runQueueWorker, daemon=True)
        uiThread = threading.Thread(target=self.runUI, daemon=True)

        queueThread.start()
        uiThread.start()

        queueThread.join()
        uiThread.join()

    def runQueueWorker(self):
        print("Queue worker running")

        tcpServer = server.Server(self.queue, Converter())
        tcpServer.run()

    def runUI(self):

        root = tkinter.Tk()
        root.columnconfigure(0, pad=3)
        root.columnconfigure(1, pad=3)
        root.rowconfigure(0, pad=3)
        root.rowconfigure(1, pad=3)
        var_cpu = tkinter.StringVar()
        var_memory = tkinter.StringVar()
        label_cpu = tkinter.Label(root, textvariable=var_cpu, relief=RAISED)
        label_memory = tkinter.Label(root, textvariable=var_memory)
        label_cpu.grid(row=0, column=0)
        label_memory.grid(row=0, column=1)

        root.update()
        while True:
            item = self.queue.get()

            if type(item) is Cpu:
                var_cpu.set(str(item.value))
                var_memory.set(str(item.value))
                root.update()
            self.queue.task_done()
